/*
Copyright 2015-2016, Manuel Bichler
WWW: <http://dbai.tuwien.ac.at/proj/lpopt/>

This file is part of lpopt.

lpopt is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lpopt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lpopt.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef HYPERGRAPHABLESTATEMENT_H_
#define HYPERGRAPHABLESTATEMENT_H_

#include <string>
#include <vector>
#include <map>

#include <htd/main.hpp>

#include "../asp-domain/asp-domain.hpp"
#include "../asp-domain/StringGenerator.hpp"

using namespace htd;
using namespace domain;

namespace lpopt
{
	class RootingOperation;

	class HypergraphableStatement
	{
	public:
		HypergraphableStatement(const Statement& toWrap, bool ignoreHeadVariables, StringGenerator& tempPredicateNameGenerator);
		~HypergraphableStatement(); // will not delete toWrap or tempPredicateNameGenerator
	public:
		IHypergraph *toHypergraph(const htd::LibraryInstance& htd);
		const Variable* getVariableById(htd::vertex_t vertex) const;
		vertex_t getIdOfVariable(const Variable& var) const;
		bool ignoreHead() const;
		void setHeadAsRoot(IMutableTreeDecomposition& decomp) const; // searches the decomp for a bag that contains all this statement's root variables and sets it as root of the decomp. If ignoreHeadVariables, do nothing.
		const Statement& stmt() const;
		StringGenerator& gen(); // returns the tempPredicateNameGenerator
		RootingOperation* createHeadRootingOperation(const LibraryInstance * managementInstance) const; // returns an operation that does what setHeadAsRoot does
	private:
		template <class VertexIterator>
		static vertex_t findContainingBag(ITreeDecomposition& decomp, vertex_t start, VertexIterator begin, VertexIterator end); // searches the part of decomp with start as its root a bag that contains all vertices in the given range. the range from begin to end must be sorted naturally. Returns 0 if not found.
	private:
		const Statement* wrapped;
		StringGenerator* stringGenerator;
		const bool ignoreHeadVariables;
		const vector<const Variable *> varlist;
		vertex_t firstVertex; // ID of first vertex in hypergraph.
		std::map<const Variable *, vertex_t, var_cmp> reverseNameMap;
	};

	class RootingOperation : public ITreeDecompositionManipulationOperation
	{ // operation that searches the decomposition for a node whose bag includes all of a certain statement's head's variables and makes this node the decomp's root node.
	public:
		RootingOperation(const HypergraphableStatement& stmt, const LibraryInstance * managementInstance);
		~RootingOperation();
		void apply(const htd::IMultiHypergraph & graph, htd::IMutableTreeDecomposition & decomposition) const;
		void apply(const htd::IMultiHypergraph & graph, htd::IMutableTreeDecomposition & decomposition, const std::vector<htd::vertex_t> & relevantVertices, std::vector<htd::vertex_t> & createdVertices, std::vector<htd::vertex_t> & removedVertices) const;
		void apply(const htd::IMultiHypergraph & graph, htd::IMutableTreeDecomposition & decomposition, const std::vector<htd::ILabelingFunction *> & labelingFunctions) const;
		void apply(const htd::IMultiHypergraph & graph, htd::IMutableTreeDecomposition & decomposition, const std::vector<htd::vertex_t> & relevantVertices, const std::vector<htd::ILabelingFunction *> & labelingFunctions, std::vector<htd::vertex_t> & createdVertices, std::vector<htd::vertex_t> & removedVertices) const;
		bool isLocalOperation(void) const;
		bool createsTreeNodes(void) const;
		bool removesTreeNodes(void) const;
		bool modifiesBagContents(void) const;
		bool createsSubsetMaximalBags(void) const;
		bool createsLocationDependendLabels(void) const;
		RootingOperation * clone(void) const;
		RootingOperation * cloneTreeDecompositionManipulationOperation(void) const;
		const LibraryInstance * managementInstance(void) const HTD_NOEXCEPT;
		void setManagementInstance(const LibraryInstance * const manager);
		const RootingOperation * cloneDecompositionManipulationOperation(void) const;
	private:
		const HypergraphableStatement* stmt;
		const htd::LibraryInstance * mgm;
		
	};
} // lpopt

#endif //HYPERGRAPHABLESTATEMENT_H_

