/*
Copyright 2015-2016, Manuel Bichler
WWW: <http://dbai.tuwien.ac.at/proj/lpopt/>

This file is part of lpopt.

lpopt is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lpopt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lpopt.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STATEMENTOPTIMIZATIONALGORITHM_H_
#define STATEMENTOPTIMIZATIONALGORITHM_H_

#include <string>
#include <list>
#include <ctime>

#include <htd/main.hpp>

#include "../asp-domain/asp-domain.hpp"
#include "../asp-domain/StringGenerator.hpp"
#include "HypergraphableStatement.hpp"

using namespace htd;
using namespace domain;

namespace lpopt
{
	class StatementOptimizationPartialSolution // represents what a decomposition's child node needs to pass to its parent in order for the parent to be able to continue optimization
	{
	public:
		StatementOptimizationPartialSolution();
		~StatementOptimizationPartialSolution(); // deeply deletes members (except ignoreHeadTags, the const Variable* elements of which are managed elsewhere)
	public:
		std::list<Statement*> createdStatements;
		ClassicalLiteral* temporary; // NULL if root
		varset ignoreHeadTags; // if head vairalbes were ignored during decomposition, this set contains all those variables that had and have to be tagged along to the root node as temporary predicates' arguments. otherwise empty/irrelevant.
	};

	class StatementOptimizationAlgorithm
	{
	public:
		StatementOptimizationAlgorithm();
		~StatementOptimizationAlgorithm();
		list<Statement*>* optimize(const ITreeDecomposition &decomposition, HypergraphableStatement &instance) const; // returns a list of statement that is the optimization of the given statement, using the given statement's given tree decomposition
	private:
		StatementOptimizationPartialSolution* evaluateNode(
				vertex_t node,
				const ITreeDecomposition &decomposition,
				HypergraphableStatement &instance,
				Statement* workingStatement, // if node==decomposition.root(), this method has memory responsibility of workingStatement
				const varset& originalStatementsVars) const;
	};
} // lpopt

#endif //STATEMENTOPTIMIZATIONALGORITHM_H_

